# coding: utf-8
from collections import OrderedDict

from django.core.exceptions import FieldDoesNotExist
from django.db.models.fields.related import ForeignKey
from django.db.models.fields.reverse_related import ForeignObjectRel


def get_field(model, name):
    for field in getattr(model, '_meta').get_fields():
        if isinstance(field, ForeignObjectRel):
            field_name = field.get_accessor_name()
        else:
            field_name = field.name

        if name == field_name:
            return field

    raise FieldDoesNotExist(name)


def get_nested_field(model, field_name):
    attr_name, _, nested_attr = field_name.partition('.')

    try:
        attr_value = model._meta.get_field(attr_name)
    except FieldDoesNotExist:
        # если это не поле, то возможно это обратная связь
        for field in model._meta.get_fields():
            if field.one_to_many and attr_name == field.get_accessor_name():
                if nested_attr:
                    attr_value = field.related_model
                else:
                    attr_value = field
                break
        else:
            raise

    if nested_attr:
        if isinstance(attr_value, ForeignKey):
            attr_value = attr_value.rel.to

        try:
            return get_nested_field(attr_value, nested_attr)
        except FieldDoesNotExist:
            raise FieldDoesNotExist(
                "{} has not field named '{}'"
                .format(model.__name__, field_name)
            )
    else:
        return attr_value


def get_columns_hierarchy(*columns):
    result = OrderedDict()
    for column in columns:
        nodes = result
        for level_name in column.split('.'):
            if level_name not in nodes:
                nodes[level_name] = OrderedDict()
            nodes = nodes[level_name]

    return result
